"""
click-man - Auto generate click documentations

This module provides a formatter for debian
man pages.
"""

from __future__ import unicode_literals


class ManPage(object):
    """
    Represent a man page

    :param str name: the name of the man page
    :param str title: the title for the man page
    """
    TITLE_KEYWORD = '.TH'
    SECTION_HEADING_KEYWORD = '.SH'
    PARAGRAPH_KEYWORD = '.PP'
    BOLD_KEYWORD = '.B'
    INDENT_KEYWORDD = '.TP'

    def __init__(self, name, title):
        #: Holds the name for the man page
        self.name = name

        #: Holds the title for the man page
        self.title = title

        #: Holds the command path for the man page
        self.command_path = self.name

        #: Holds the short help for the man page
        self.short_help = ''

        #: Holds the synopsis for the man page
        self.synopsis = ''

        #: Holds the description for the man page
        self.description = ''

        #: Holds a list of tuple options for the man page
        #  the first item in the tuple are the option switches
        #  and the second one is the option's description
        self.options = []

        #: Holds the commands for the man page
        self.commands = []

    def __str__(self):
        """
        Show the string representation for
        the man page.
        """
        lines = []

        # write title and footer
        lines.append('{0} {1} 1 "{2}" "{3} Manual"'.format(
            self.TITLE_KEYWORD, self.title, '21-Feb-1994', self.name))

        # write name section
        lines.append('{0} NAME'.format(self.SECTION_HEADING_KEYWORD))
        lines.append(r'{0} \- {1}'.format(self.title, self.short_help))

        # write synopsis
        lines.append('{0} SYNOPSIS'.format(self.SECTION_HEADING_KEYWORD))
        lines.append('{0} {1}'.format(self.BOLD_KEYWORD, self.command_path))
        lines.append(self.synopsis.replace('-', r'\-'))

        # write the description
        lines.append('{0} DESCRIPTION'.format(self.SECTION_HEADING_KEYWORD))
        lines.append(self.description)  # FIXME: replace empty lines with PARAGRAPH_KEYWORD

        # write the options
        if self.options:
            lines.append('{0} OPTIONS'.format(self.SECTION_HEADING_KEYWORD))
            for option, description in self.options:
                lines.append(self.INDENT_KEYWORDD)
                lines.append(option.replace('-', r'\-'))
                lines.append(description)

        # write commands
        if self.commands:
            lines.append('{0} COMMANDS'.format(self.SECTION_HEADING_KEYWORD))
            for name, description in self.commands:
                lines.append(self.INDENT_KEYWORDD)
                lines.append(name)
                lines.append(description)

        return '\n'.join(lines)



"""
click-man - Auto generate click documentations
"""

import os

import click

from .man import ManPage


def generate_man_page(ctx):
    """
    Generate documentation for the given command.

    :param Command command: the command to generate the documentation for.
    """
    man_page = ManPage(ctx.command_path, ctx.command_path)
    man_page.command_path = ctx.command_path
    man_page.short_help = ctx.command.short_help
    man_page.description = ctx.command.help
    man_page.synopsis = ' '.join(ctx.command.collect_usage_pieces(ctx))
    man_page.options = [x.get_help_record(None) for x in ctx.command.params if isinstance(x, click.Option)]

    commands = getattr(ctx.command, 'commands', None)
    if commands:
        man_page.commands = [(k, v.short_help) for k, v in commands.items()]
    return str(man_page)


def write_man_pages(name, cli, parent_ctx=None, target_dir=None):
    """
    Generate man page files recursively
    for the given click cli function.

    :param str name: the cli name
    :param cli: the cli instance
    """
    ctx = click.Context(cli, info_name=name, parent=parent_ctx)

    man_page = generate_man_page(ctx)
    path = '{0}.man'.format(ctx.command_path.replace(' ', '-'))
    if target_dir:
        path = os.path.join(target_dir, path)

    with open(path, 'w+') as f:
        f.write(man_page)

    commands = getattr(cli, 'commands', {})
    for name, command in commands.items():
        write_man_pages(name, command, parent_ctx=ctx, target_dir=target_dir)



import os
from distutils.core import Command
from distutils.errors import DistutilsSetupError
from pkg_resources import EntryPoint

from click_man.core import write_man_pages


__all__ = ['man_pages']



class man_pages(Command):
    description = 'distutils command to generate man pages'

    user_options = [
        ('target=', 't', 'Target location for the man pages')
    ]
    boolean_options = []

    def initialize_options(self):
        self.target = os.path.join(os.getcwd(), 'man')

    def finalize_options(self):
        self.target = os.path.abspath(self.target)

        # create target directory if it does not exist yet
        try:
            os.makedirs(self.target)
        except OSError:
            pass

    def run(self):
        eps = EntryPoint.parse_map(self.distribution.entry_points or '')

        if 'console_scripts' not in eps or not eps['console_scripts']:
            raise DistutilsSetupError('No entry points defined in setup()')

        console_scripts = [(k, v) for k, v in eps['console_scripts'].items()]
        # Only generate man pages for first console script
        # FIXME: create own setup() attribute for CLI script configuration
        name, entry_point = console_scripts[0]

        self.announce('Load entry point {0}'.format(name), level=2)
        cli = entry_point.resolve()
        self.announce('Generate man pages for {0}'.format(name), level=2)
        write_man_pages(name, cli, target_dir=self.target)



from setuptools import setup, find_packages

setup(
    name='click-man',
    version='0.1.0',
    license='MIT',
    author='Timo Furrer',
    author_email='tuxtimo@gmail.com',
    install_requires=[
        'click'
    ],
    packages=find_packages()
)
